/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-AE
  * @brief          : Main program body
  *    
  *          This file provides example code for USART RS485 DE function.
  * LED2 blink period 1S.For details, see "readme.txt"    
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
 */  
#include "system_define.h"

#define REV_STR "2023-02-16"
#define HW_buff "\r\n\r\n*****UART RS485 Example*****\r\n*****rev:"REV_STR" "__TIME__ 

#define LED_BLINK_HALF_PRE        5    /* uint 100ms */

uint16_t ADC_Result_buff[ADC_CH_NUM] ={0};
uint16_t ADC_Voltage_buff[ADC_CH_NUM] ={0};

/**
  * @brief  Main program
  * @param  None
  * @retval None
 */
int main(void)
{
  uint8_t LED_count;
  uint16_t run_count=0;
  ADCErrorStatus ADC_status = ADC_NO_ERROR;

  sysclk_init();        /* sysclk 48M,HSI/2 *12;if not call this function,default HSI 8M */ 
  SystemCoreClockUpdate();
  
  sys_tick_init();
  gpio_init();

  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);  /* uart clk HSI 8M */ 
  usart1_rs485_init(RS485_BAUD_RATE);

  adc_init(); 
  virtualUart_Init();   /*virtual uart pin and timer initialization*/

  Debug_Log(HW_buff);   /*example information output*/ 
  Debug_Log("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock);
  Debug_Log("\r\n--Inf:RS485 address %d.",SLAVE_ADD);

  ADC_status = Activate_ADC();    /* Calibration and enable ADC */
  if(ADC_status !=ADC_NO_ERROR ) {
    ADC_ERROR(ADC_status);
    ADC_status =ADC_NO_ERROR;
  }
  Start_100MS_SF_Timer();
  while (1) {
    Recv_DataAnalysis();          /* RS485 data ananlysis and dispose*/

    if(Get_State_100MS_SF_Timer() == SF_TIMER_OV) {
      Start_100MS_SF_Timer();

      ADC_status = Get_ADC_Conversion_Result(ADC_Result_buff);  /* interval 100ms start ADC convert*/    
      if(ADC_status !=ADC_NO_ERROR ) {
        ADC_ERROR(ADC_status);
        ADC_status =ADC_NO_ERROR;
      }
      else {                                                    /* get adc result OK,calc to mv and store to buff*/
        ADC_Voltage_buff[0] = __MS32_ADC_CALC_DATA_TO_VOLTAGE(USER_VDDA, ADC_Result_buff[0], MS32_ADC_RESOLUTION_12B);
        ADC_Voltage_buff[1] = __MS32_ADC_CALC_DATA_TO_VOLTAGE(USER_VDDA, ADC_Result_buff[1], MS32_ADC_RESOLUTION_12B);
      } 
      /* LED and printf for running indicate*/
      LED_count++;
      if( LED_count ==LED_BLINK_HALF_PRE) {  
        run_count++;
        Debug_Log("\r\n--Inf:run %dms count %d.",LED_BLINK_HALF_PRE*100,run_count);
        LED_count =0;
        LED2_TOGGLE();
      }
    }
  }
}

/**
  * @brief  SystemClock_Config HSI/2 to PLL*12 sysclk 48M
  * @param  None
  * @retval None
 */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  /* get and clear reset flag */
  MS32_RCC_GetResetFlag(ENABLE);
  
  /* osc config HSI On,others OFF*/ 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_OFF;
  osc_cfg.LSIState = MS32_RCC_LSI_OFF;
  osc_cfg.LSEState = MS32_RCC_LSE_OFF;
  MS32_RCC_OscConfig(&osc_cfg);
  
  /* pll config HSI/2*12=48M*/ 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;       /* HSI/2 4M */
  pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;
  pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
  MS32_RCC_PllConfig(&pll_cfg);
  
  /* sysclk config */ 
  clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}

/******************************** END OF FILE ******************************** */
