/**
  ******************************************************************************
  * @file     Virtual_UART.c
  * @author   sinomcu.com-AE
  * @brief    
  *    
  *         
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
 */

#include "stdio.h"  
#include "ms32f0xx.h"
#include "user_config.h"
#include "Virtual_UART.h"

#define ONEBYTE_BITNUM            8                             /*one byte to bit*/
#define UINT16T_MAX               65535

#define IO_USART_SENDDELAY_TIME   (1000000/VIRTUAL_UART_BAUD)   /*unit us,VIRTUAL_UART_BAUD define by user_config.h*/
#define virtualUart_PinHigh()     MS32_GPIO_SetOutputPin(VirtualUART_Port,VirtualUART_TxPIN)
#define virtualUart_PinLow()      MS32_GPIO_ResetOutputPin(VirtualUART_Port,VirtualUART_TxPIN)

/**
  * @brief  Timer17 Initialization,for virtual uart send
  *         1us each count,UIF 65536us
  * @param  None
  * @retval None
  */
void timer17_init(void) {
  MS32_TIM_InitTypeDef TimerInitStruct = {0};
  MS32_APB1_GRP2_EnableClock(MS32_APB1_GRP2_PERIPH_TIM17);

  /* count up,virtual uart baud */
  /* Prescaler 16bit,CK_CNT=TIMclk/(PSD+1) */
  /* UIF:16bit,Autoreload,CK_CNT*(Autoreload+1),(0 ~ Autoreload) */
  MS32_TIM_StructInit(&TimerInitStruct);
  TimerInitStruct.CounterMode = MS32_TIM_COUNTERMODE_UP;
  TimerInitStruct.Prescaler   = __MS32_TIM_CALC_PSC(SystemCoreClock,1000000);  /* unit Hz,div sysclk to  CNT clk:1Mhz */           
  TimerInitStruct.Autoreload  = UINT16T_MAX;                              /* 65535us */
  TimerInitStruct.ClockDivision = MS32_TIM_CLOCKDIVISION_DIV1;            /* for tDTS */
  MS32_TIM_Init(TIM17, &TimerInitStruct);

  MS32_TIM_EnableARRPreload(TIM17);
  MS32_TIM_SetClockSource(TIM17, MS32_TIM_CLOCKSOURCE_INTERNAL);          /* clk from RCC:PCLK <-- AHP_RRE=1,sysclk <-- HSI8M or PLL */
  MS32_TIM_SetTriggerOutput(TIM17, MS32_TIM_TRGO_RESET);                  /* for Master/Slave mode synchronization */
  MS32_TIM_DisableMasterSlaveMode(TIM17);                                 /* disable the Master/Slave mode. */
  MS32_TIM_EnableCounter(TIM17);                                          /* start timer */
}

/**
  * @brief  delay n us,block
  * @param  usTime,uint16_t,unit us
  * @retval None
  */
void block_delayUs(volatile uint16_t usTime) { 
  uint32_t countTmp;
 
  countTmp = MS32_TIM_GetCounter(TIM17);    /* get timer17 count current*/
  if( (countTmp + usTime) <= 65535) {       /* timer to max enough for delay*/
    while( (MS32_TIM_GetCounter(TIM17) - countTmp) < usTime ) { ; }
  }
  else {                                    /* timer to max not enough for delay*/
    MS32_TIM_SetCounter(TIM17,0);
    while( MS32_TIM_GetCounter(TIM17) < usTime ) { ; }
  }
}

/**
  * @brief  initialization virtual uart send pin,output high
  * @param  None
  * @retval None
 */
void virtualUart_PinInit(void) {
  MS32_GPIO_InitTypeDef  GpioInitStruct;
  
  /* TX pin out pullup */ 
  MS32_GPIO_StructInit(&GpioInitStruct);  
  GpioInitStruct.Pin =  VirtualUART_TxPIN;
  GpioInitStruct.Mode = MS32_GPIO_MODE_OUTPUT;
  GpioInitStruct.OutputType = MS32_GPIO_OUTPUT_PUSHPULL;
  GpioInitStruct.Speed = MS32_GPIO_SPEED_HIGH;
  GpioInitStruct.Pull = MS32_GPIO_PULL_UP;
  GpioInitStruct.Alternate = MS32_GPIO_AF_0;
  MS32_GPIO_Init(VirtualUART_Port,&GpioInitStruct);
}

/**
  * @brief  initialization virtual uart:pin and timer
  * @param  None
  * @retval None
 */
void virtualUart_Init(void) {
  virtualUart_PinInit();   
  timer17_init();
}

/**
  * @brief  send one byte by virtual uart
  * @param  None
  * @retval None
 */
void virtualUart_SendByte(uint8_t sendData)
{
  uint8_t count=0, sendBit=0;
 
  /*start bit*/
  virtualUart_PinLow();
  block_delayUs(IO_USART_SENDDELAY_TIME);  
  
  /*send Byte*/
  for(count = 0; count < ONEBYTE_BITNUM; count++)
  {
    sendBit  = (sendData >> count) & 0x01;  /*LSB to MSB*/
 
    if(sendBit) {
      virtualUart_PinHigh();
      block_delayUs(IO_USART_SENDDELAY_TIME);  
    } 
    else {
      virtualUart_PinLow();
      block_delayUs(IO_USART_SENDDELAY_TIME);   
    }
  }
  
  /*stop bit*/
  virtualUart_PinHigh();
  block_delayUs(IO_USART_SENDDELAY_TIME);  
}

#if defined ( __GNUC__ ) && !defined (__clang__)      /* GCC, not Keil MDK AC6 */
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  fd:file descriptor,ptr:send buff pointer,len:send byte number
  * @retval None
 */
int _write (int fd, char *ptr, int len) {
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  uint16_t send_data_len=len;

  for(; send_data_len >0;) {
      virtualUart_SendByte(*ptr++);
      send_data_len--;
    }
  }
#endif
  return len;
}  
#endif
#if defined ( __CC_ARM ) || defined (__clang__)       /* Keil MDK, AC5|| AC6 */
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  ch:send byte,file:library,file pointer
  * @retval None
 */
int fputc(int ch, FILE *f) {
#if(defined(EN_PRINT_UART) && EN_PRINT_UART) 
  virtualUart_SendByte(ch);
#endif
  return ch;
}
#endif
#ifdef __ICCARM__  /* IAR */
/**
  * @brief  Retargets the C library printf function to the USART.
  * @param  ch:send byte
  * @retval None
 */ 
int putchar(int ch) {
#if(defined(EN_PRINT_UART) && EN_PRINT_UART)  
  virtualUart_SendByte(ch);
#endif
  return ch;
}
#endif
/******************************** END OF FILE ******************************** */
